if not (Klappa) then return; end;
local _G = getfenv(0);

local Klappa = Klappa;
local config = {};

local AceOO = AceLibrary("AceOO-2.0");
local L = AceLibrary("AceLocale-2.2"):new("Klappa");

Klappa.Bar = AceOO.Class("AceEvent-2.0");

function Klappa.Bar.prototype:init(index)
	Klappa.Bar.super.prototype.init(self);
	
	config = Klappa.config.bars;
	self.index = index;
	self:LoadDefaults();
	self:AddOptions();
	self:CreateBar();
	self.root.buttons = {};
	if(config[self.index].buttons == nil) then
		config[self.index].buttons = {};
		config[self.index].buttons[1] = {};
		config[self.index].buttons[1].id = 1;
		config[self.index].buttons[1].popups = {};
		self.root.buttons[1] = Klappa.MainButton:new(1,"Row"..1, 1, self);
	else
		for i, button in pairs (config[self.index].buttons) do
			self.root.buttons[i] = Klappa.MainButton:new(i,"Row"..i, button.id, self);
		end
	end
	self:LoadPosition();
	self:UpdateLayout();
	
	Klappa:Debug("Bar "..index.." created");
end

function Klappa.Bar.prototype:CreateBar()
	self.root = CreateFrame("Frame", "KlappaBar"..self.index, UIParent, "SecureStateHeaderTemplate");
	self.root:SetPoint("CENTER", UIParent, "CENTER", 0, 0);
	self.root:RegisterForDrag("LeftButton");
	self.root:EnableMouse(false);
	self.root:SetMovable(true);
	self.root:SetClampedToScreen(true);
	
	self.root.texture = self.root:CreateTexture();
	self.root.texture:SetTexture(0,0,0.5,0);
	self.root.texture:SetAllPoints(self.root);
end


function Klappa.Bar.prototype:UpdateLayout()
	local rootx, rooty = 0, 0;
	local isVert, isRtDn = false, false;
	local size = config[self.index].size;
	local orientation = config[self.index].orient;
	if (orientation == "horzdown") then
		isRtDn = true;
	elseif (orientation == "vertleft") then
		isVert = true;
	elseif (orientation == "vertright") then
		isVert = true;
		isRtDn = true;
	end
	if (isVert) then
		rootx = size;
	else
		rooty = size;
	end
	for idx, buttonClass in pairs(self.root.buttons) do
		local x, y = 0, 0;
		local padding = config[self.index].padding;
		if (isVert) then
			y = (-size-padding)*(buttonClass.index - 1);
			rooty = rooty + size + padding;
		else
			x = (size+padding)*(buttonClass.index - 1);
			rootx = rootx + size + padding;
		end
		buttonClass:UpdateLayout(x, y, isVert, isRtDn);
	end
	self.root:SetScale(config[self.index].buttonScale);
	self.root:SetWidth(rootx);
	self.root:SetHeight(rooty);
end

function Klappa.Bar.prototype:ToggleLock()
	if (config[self.index].locked) then
		self.root:EnableMouse(true);
		self.root:SetScript("OnDragStart", function() self:StartDrag(); end);
		self.root:SetScript("OnDragStop", function() self:StopDrag(); end);
		self.root.texture:SetTexture(0, 0, 0.5, 0.5)
		self.root:SetFrameLevel(4);
		config[self.index].locked = false;
		-- Show all popups
		for idx, buttonClass in pairs(self.root.buttons) do
			buttonClass:ShowPops();
		end
	else
		self.root:EnableMouse(false);
		self.root:SetScript("OnDragStart", nil);
		self.root:SetScript("OnDragStop", nil);
		self.root.texture:SetTexture(0, 0, 0.5, 0);
		self.root:SetAttribute("state", 0)
		config[self.index].locked = true;
	end		
end

function Klappa.Bar.prototype:ToggleLockButtons()
	if(config[self.index].lockButtons) then
		config[self.index].lockButtons = false;
	else
		config[self.index].lockButtons = true;
	end
end

function Klappa.Bar.prototype:StartDrag()
	self.root:StartMoving();
end

function Klappa.Bar.prototype:StopDrag()
	self.root:StopMovingOrSizing();
	self:SavePosition();
end

function Klappa.Bar.prototype:LoadPosition()
	local x, y, s = config[self.index].rootX, config[self.index].rootY, self.root:GetEffectiveScale();
	
	self.root:ClearAllPoints();
	self.root:SetPoint(x and "TOPLEFT" or "CENTER", UIParent, x and "BOTTOMLEFT" or "CENTER", x or 0, y or 0);
end

function Klappa.Bar.prototype:SavePosition()
	local x, y, s = self.root:GetLeft(), self.root:GetTop(), self.root:GetEffectiveScale();
	
	config[self.index].rootX = x;
	config[self.index].rootY = y;
end


function Klappa.Bar.prototype:AddMainButton()
	idx = config[self.index].numberButtons + 1;
	local nextid = self.root.buttons[idx-1].id + self.root.buttons[idx-1].button.popups + 1;
	if(nextid >= 120) then nextid = 120; end
	config[self.index].buttons[idx] = {};
	config[self.index].buttons[idx].id = nextid or config[self.index].id;
	config[self.index].buttons[idx].popups = {};
	
	self.root.buttons[idx] = Klappa.MainButton:new(idx,"Row"..idx, nextid or config[self.index].id, self);
	config[self.index].numberButtons = idx;
	self:UpdateLayout();
end

function Klappa.Bar.prototype:DelMainButton()
	index = config[self.index].numberButtons;
	if (index == 0 or index == nil) then return end;
	self.root.buttons[index].button:Hide();
	self.root.buttons[index].button.popups = nil;
	self.root.buttons[index].button = nil;
	self.root.buttons[index] = nil;
	
	config[self.index].buttons[index] = nil;
	Klappa.options.args["Bar"..self.index].args["Main"..index] = nil;
	config[self.index].numberButtons = index-1;
	self:UpdateLayout();
end

function Klappa.Bar.prototype:LoadDefaults()
	--if(config[self.index] == nil) then return end
	if (config[self.index].numberButtons == nil) then config[self.index].numberButtons = 1; end
	if (config[self.index].orient == nil) then config[self.index].orient = "vertleft"; end
	if (config[self.index].buttonScale == nil) then config[self.index].buttonScale = 1; end
	if (config[self.index].popupScale == nil) then config[self.index].popupScale = 1; end
	if (config[self.index].alpha == nil) then config[self.index].alpha = 1; end
	if (config[self.index].popupAlpha == nil) then config[self.index].popupAlpha = 1; end
	if (config[self.index].locked == nil) then config[self.index].locked = true; end
	if (config[self.index].lockButtons == nil) then config[self.index].lockButtons = true; end
	if (config[self.index].padding == nil) then config[self.index].padding = 1; end
	if (config[self.index].size == nil) then config[self.index].size = 36; end
	if (config[self.index].tooltip == nil) then config[self.index].tooltip = true; end
	if (config[self.index].id == nil) then config[self.index].id = 20; end
end

function Klappa.Bar.prototype:AddOptions()
	Klappa.options.args["Bar"..self.index] = {
		name = L["Bar "]..self.index,
		desc = L["Options for the bar"],
		type = "group",
		args = {}
		};
	Klappa.options.args["Bar"..self.index].args = {
		lock = {
				name = L["Lock the frame"],
				desc = L["Lock the frame"],
				type = "toggle",
				order = 1,
				get = function() return config[self.index].locked end,
				set = function(v) self:ToggleLock() end,
				disabled = function() return InCombatLockdown() end,
			},
		lockbuttons = {
			name = L["Button Lock"],
			desc = L["Lock the buttons"],
			type = "toggle",
			order = 2,
			get = function() return config[self.index].lockButtons end,
			set = function(v) self:ToggleLockButtons() end,
		},
		tooltip = {
			name = L["Tooltip"],
			desc = L["Enable/disable the tooltips"],
			type = "toggle",
			order = 3,
			get = function() return config[self.index].tooltip end,
			set = function(v) config[self.index].tooltip = v end,
		},
		padding = {
			type = "range",
			name = L["Padding"],
			desc = L["Set the padding of the buttons"],
			order = 4,
			step = 0.05,
			min = 0.05,
			max = 25.0,
			isPercent = false,
			get = function() return config[self.index].padding end,
			set = function(v)
				config[self.index].padding = v;
				self:UpdateLayout();
			end			
		},
		mainScale = {
			type = "range",
			name = L["Main Scale"],
			desc = L["Scale the buttons"],
			order = 5,
			step = 0.05,
			min = 0.25,
			max = 5.0,
			isPercent = false,
			get = function() return config[self.index].buttonScale; end,
			set = function(v)
				config[self.index].buttonScale = v; self.root:SetScale(v); end
		},	
		orientation = {
			type = "text",
			name = L["Orientation"],
			desc = L["Set the orientation of the Klappa Bar."],
			order = 6,
			get = function() return config[self.index].orient; end,
			set = function(v) config[self.index].orient = v; self:UpdateLayout(); end,
			validate = { 
				["horzup"] = L["Horizontal, Grow Up"],
				["horzdown"] = L["Horizontal, Grow Down"],
				["vertright"] = L["Vertical, Grow Right"],
				["vertleft"] = L["Vertical, Grow Left"],
			},
		},
		add = {
			name = L["Add main button"],
			desc = L["Add a new main button"],
			type = "execute",
			func = function() self:AddMainButton() end,
		},
		del = {
			name = L["Delete main button"],
			desc = L["Delete last main button"],
			type = "execute",
			func = function() self:DelMainButton() end,
		},
	}
end
