if not (Klappa) then return; end;
local _G = getfenv(0);

local Klappa = Klappa;
local config = {};

local AceOO = AceLibrary("AceOO-2.0");
local L = AceLibrary("AceLocale-2.2"):new("Klappa");

Klappa.MainButton = AceOO.Class(Klappa.Button);


function Klappa.MainButton.prototype:init(idx, type, id, bar)
	Klappa.MainButton.super.prototype.init(self, idx, type, id, bar);
	
	config = Klappa.config.bars;
	self:CreateMainButton();
	self:AddOptions();
	
	self.popupButtons = {};
	self.button.popupButtons = self.popupButtons;
	self.button.popups = 0;
	
	if(config[self.barid].buttons[idx] ~= nil) then
		for i, popup in pairs (config[self.barid].buttons[idx].popups) do
			self.button.popups = self.button.popups + 1;
			self.button.popupButtons[i] = Klappa.PopUpButton:new(self, i, type, popup.id, bar);
		end
	else
		config[self.barid].buttons[idx] = {};
		config[self.barid].buttons[idx].id = id;
		config[self.barid].buttons[idx].popups = {};
	end
	
	self:LoadSpell();
	Klappa:Debug("MainButton: "..self.button:GetName().." created");
end

function Klappa.MainButton.prototype:CreateMainButton()
	local name = "KlappaBar"..self.barid..self.type.."Button1";
	self.header = CreateFrame("Frame", "Klappa"..self.type.."Frame", self.bar.root, "SecureStateHeaderTemplate");
	self.button = CreateFrame("CheckButton", name, self.header, "SecureAnchorEnterTemplate,SecureActionButtonTemplate, ActionButtonTemplate");
	self.button.name = name;
	self.button:SetPoint("TOPLEFT", self.header, "TOPLEFT", 0, 0);
	self.button:SetWidth(config[self.barid].size);
	self.button:SetHeight(config[self.barid].size);
	self.button.class = self;
	self.button.id = self.id;
	self.button.index = self.index;
	
	self.button:HookScript("OnEnter", function() self:ShowTooltip(self.button); end);
	self.button:HookScript("OnLeave", function() self:HideTooltip(self.button); end);
	
	self:ButtonProperties();
	self:SetAttributes();
end

function Klappa.MainButton.prototype:SetAttributes()
	self.button:SetAttribute("childstate-onenter", "show"..self.type);
	self.button:SetAttribute("childstate-onleave", "hide"..self.type);
	self.button:SetAttribute("anchorchild", self.bar.root);

	self.header:SetAttribute("addchild", self.button);
	self.bar.root:SetAttribute("statemap-anchor-show"..self.type, "*:"..self.index);
	self.bar.root:SetAttribute("statemap-anchor-hide"..self.type, ";");
	self.bar.root:SetAttribute("delaystatemap-anchor-hide"..self.type, self.index..":0");
	self.bar.root:SetAttribute("delaytimemap-anchor-hide"..self.type,  self.index..":0.5");
	self.bar.root:SetAttribute("delayhovermap-anchor-hide"..self.type, self.index..":true");
end

function Klappa.MainButton.prototype:UpdateLayout(x, y, isVert, isRtDn)
	self.header:ClearAllPoints();
	self.header:SetPoint("TOPLEFT", self.bar.root, "TOPLEFT", x, y);
	local scale = config[self.barid].buttonScale;
	local size = config[self.barid].size;
	for idx, popButtonClass in pairs(self.button.popupButtons) do
		local x, y = 0, 0;
		local index = idx;
		local padding = config[self.barid].padding;
		if (isVert) then
			x = (size+padding)*index - size;
			if (not isRtDn) then
				x = x*(-1);
			end
		else
			y = (-size-padding)*index + size;
			if (not isRtDn) then
				y = y*(-1);
			end
		end
		popButtonClass:UpdateLayout(isVert, isRtDn, x, y);
	end
	self.button:SetAlpha(config[self.barid].alpha);
end

function Klappa.MainButton.prototype:ShowPops()
	for idx, popupButtonClass in pairs (self.button.popupButtons) do
		popupButtonClass.button:Show();
	end
end

function Klappa.MainButton.prototype:AddOptions()
	Klappa.options.args["Bar"..self.barid].args["Main"..self.index] = {
		name = L["Main button "]..self.index,
		desc = L["Options for the button"],
		type = "group",
		args = {}
		};
	Klappa.options.args["Bar"..self.barid].args["Main"..self.index].args = {
		changeId = {
			name = L["Change main button-ID"],
			desc = L["Change the ID of this mainbutton"],
			type = "range",
			order = 1,
			step = 1,
			min = 1,
			max = 120,
			isPercent = false,
			get = function() return self.button.id; end,
			set = function(v) self:ChangeID(v); end			
		},
		addPopup = {
			name = L["Add popup button"],
			desc = L["Add a new popup button"],
			type = "execute",
			order = 2,
			func = function() self:AddPopup() end
		}, 
		delPopup = {
			name = L["Delete popup button"],
			desc = L["Delete the last popup button"],
			type = "execute",
			order = 3,
			func = function() self:DelPopup() end
		}
	}
end

function Klappa.MainButton.prototype:ChangeID(id)
	self.button.id = id;
	self:LoadSpell();
	config[self.barid].buttons[self.index].id = id;
end

function Klappa.MainButton.prototype:AddPopup()
	self.button.popups = self.button.popups + 1;
	local popupid = self.button.id + self.button.popups;
	if(popupid >= 120 or popupid == nil) then popupid = 120; end
	self.button.popupButtons[self.button.popups] = Klappa.PopUpButton:new(self, self.button.popups, self.type, popupid, self.bar);
	config[self.barid].buttons[self.index].popups[self.button.popups] = {};
	config[self.barid].buttons[self.index].popups[self.button.popups].id = popupid;
	self.bar:UpdateLayout();
end

function Klappa.MainButton.prototype:DelPopup()
	local lastpopup = self.button.popups;
	if(lastpopup <= 0) then lastpopup = 0; return end;
	self.button.popupButtons[lastpopup].button.icon:Hide();
	self.button.popupButtons[lastpopup].button:SetAttribute("showstates", "!*");
	self.button.popupButtons[lastpopup].button:Hide();
	self.button.popupButtons[lastpopup].button = nil;
	self.button.popupButtons[lastpopup] = nil;
	config[self.barid].buttons[self.index].popups[lastpopup] = nil;
	self.button.popups = lastpopup - 1;
	self.bar:UpdateLayout();
	
	Klappa.options.args["Bar"..self.barid].args["Main"..self.index].args[lastpopup] = nil;
end

